@@ -3,7 +3,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $logger);
-$VERSION = '1.16_02';
+$VERSION = '1.17_02';
 
 use File::Basename;
 use File::Spec::Functions qw(catfile);
@@ -58,7 +58,7 @@ sub get_task
 		
 		my $Config = $Notes->{config};
 		
-		$logger->warn( "Child [$$] processing $dist\n" );
+		$logger->info( "Child process for $basename starting\n" );
 			
 		my $Indexer = $Config->indexer_class || 'MyCPAN::Indexer';
 		
@@ -70,28 +70,31 @@ sub get_task
 			exit 255;
 			}
 		
-		local $SIG{ALRM} = sub { die "alarm\n" };
+		local $SIG{ALRM} = sub { die "alarm rang for $basename!\n" };
 		alarm( $Config->alarm || 15 );
 		my $info = eval { $Indexer->run( $dist ) };
-	
+		alarm 0;
+
 		unless( defined $info )
 			{
-			$logger->error( "run failed: $@" );
-			return;
+			$logger->error( "run failed for $basename: $@" );
+			$info = bless {}, $Indexer; # XXX TODO make this a real class
+			$info->setup_dist_info( $dist );
+			$info->setup_run_info;
+			$info->set_run_info( qw(completed 0) );
+			$info->set_run_info( error => $@ );
 			}
 		elsif( ! eval { $info->run_info( 'completed' ) } )
 			{
 			$logger->error( "$basename did not complete\n" );
 			$class->_copy_bad_dist( $Notes, $info ) if $Config->copy_bad_dists;
 			}
-			
-		alarm 0;
-				
+
 		$class->_add_run_info( $info, $Notes );
 		
 		$Notes->{reporter}->( $Notes, $info );
 		
-		$logger->debug( "Child [$$] process done" );
+		$logger->debug( "Child process for $basename done" );
 		
 		1;
 		};
@@ -167,7 +170,7 @@ sub _add_run_info
 	$info->set_run_info( $_, $Config->get( $_ ) ) 
 		foreach ( $Config->directives );
 	
-	$info->set_run_info( 'uuid', $Config->UUID ); 
+	$info->set_run_info( 'uuid', $Notes->{UUID} ); 
 
 	$info->set_run_info( 'child_pid',  $$ ); 
 	$info->set_run_info( 'parent_pid', getppid ); 
